
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     insertion d'une image dans la figure                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function insertion2(handles)

global Qr

   uiload;
   % Si le fichier ouvert est correct
   if exist('Br','var'),

      br=Qr;
      axes(handles.axes1);
      bouton=1;
      % Calcul du milieu du rectangle (dans lequel s'inscrit la forme de la texture)
      [taillex,tailley]=size(Br);
      milieux=round(taillex/2);          
      milieuy=round(tailley/2);
      [y,x,bouton]=ginput(1);
      
      % tant que non clic droit
      while bouton~=3,
      
         x=floor(x);
         y=floor(y);
         xmin=x-milieux;
         ymin=y-milieuy;
         xmax=taillex+x-milieux-1;
         ymax=tailley+y-milieuy-1;  
         r=[1 taillex 1 tailley];
         
         % Cas particulier (si la texture inseree sort de la texture courante)
         if xmax>size(br,1),
            r(2)=r(2)-(xmax-size(br,1)); 
            xmax=size(br,1);
         end 
         if ymax>size(br,2),
            r(4)=r(4)-(ymax-size(br,2)); 
            ymax=size(br,2);
         end
         if xmin<=0,
            r(1)=r(1)-xmin+1; 
            xmin=1;     
         end
         if ymin<0,
            r(3)=r(3)-ymin+1; 
            ymin=1;     
         end
         
         % On additionne et on affiche de la texture
         br(xmin:xmax,ymin:ymax)=Qr(xmin:xmax,ymin:ymax)+Br(r(1):r(2),r(3):r(4));
         afficher(br);
         % Remise à jour de Qr
         Qr=br;
         [y,x,bouton]=ginput(1);
      end
  else
    messagebox('Le fichier ouvert n''a pas le bon format! ',...
               '*************************************************************');
   
   end  
