% CALCUL DES POINTS D'UNE COURBE SPLINE DE POINTS DE CONTROLE DONNES
% 
% Entrees :
%   -> spline    : points de controle de la spline (repeter le dernier
%                  point)
%   -> nb_points : nombre de points a calculer entre deux points de
%                  controle
%
% Sorties :
%   -> courbe    : points [x,y] de la courbe calcules
%
% Exemple :
%
%  spline=[-4 -3;-1 -1.5;1 -4;1 -1;4 0;1 1;1 4;-1 1.5;-4 3;-2 0;-4 -3];
%  courbe=points_spline(spline,100);
%  plot(courbe(:,1)',courbe(:,2)');
%