
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     insertion d'une image dans la figure                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function insertion1(handles)

global Qr va zones x_max y_max r_min dx NG

    uiload;
    
   % Si le fichier ouvert est correct 
   if exist('zones_s','var'),
       
      messagebox('***************************************************',...
                 'Insertion d une partie de texture');
      tic;
      [va,zones]=selection_insertion(va,zones,va_s,zones_s);
  
      % reconstruction de la texture
      cadre=[0,0,x_max,y_max];
      [Qr,va,zones]=zcid2d(cadre,dx,r_min,r_min,va,zones);
      temps=toc;
      messagebox(sprintf('temps de calcul : %f s',temps),'','');
      NG=niveaux_gris(Qr);
      imagesc(NG);
      axis image
   else
      messagebox('Le fichier ouvert n''a pas le bon format! ',...
                 '*************************************************************');
   end