
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                 Zoom arriere                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function zoom_arriere()

global Qr va zones x_max y_max r_min dx tailles ng_min ng_max choix_mapping

   choix_mapping=0;
   
   % Si on peut depiler
   if (length(tailles)~=0)
      [tailles,Qr,va,x_max,y_max,r_min,dx,ng_min,ng_max,zones]=pile(tailles);
      afficher(Qr);
   else
      messagebox('vous ne pouvez plus faire de zoom arriere','');     
   end