
function messagebox(varargin);

% Affichage des messages dans la boite m
% Utilise les variables globales 
% HandleBox  handle de la fenetre d'affichage
% Nbligne  nombre de ligne de la fenetre d'affichage

   global HandleBox Nbligne

   if (isempty(HandleBox) | isempty(Nbligne))
      HandleBox=findobj('tag','messagebox');
      Nbligne=100;
      data=cell(Nbligne,1);
   end    
   if (nargin==1 & varargin{:}=='clear')
      text(1:Nbligne)=cell(Nbligne,1);
   else    
      nb_message=nargin;
      text=get(HandleBox,'String');
      text(1:Nbligne-nb_message)=text(nb_message+1:Nbligne);
      text((Nbligne-nb_message+1):Nbligne)={varargin{:}};
   end  
   set(HandleBox,'String',text);
   set(HandleBox,'ListboxTop',Nbligne);
 
   return;
