% GESTION DE LA PILE DES DONNEES
%
% Prototype de l'empilement :
%
%   tailles=pile(tailles,Qr,va,x_max,y_max,r_min,dx,ng_min,ng_max,zones);
%
% Prototype du depilement :
%
%   [tailles,Qr,va,x_max,y_max,r_min,dx,ng_min,ng_max,zones]=pile(tailles);
%
% Arguments :
%   -> Qr            : matrice de texture
%   -> va            : variables aleatoires utilisees pour la synthese
%   -> x_max,y_max   : taille de la matrice de texture à generer
%   -> r_min         : resolution de la cascade
%   -> dx            : pas d'espace
%   -> ng_min,ng_max : niveau de gris minimum et maximum de l'image
%   -> zones         : zones presentes dans la texture
%
%   -> tailles       : vecteur des tailles cumulees des 9 arguments
%                      precedents (avant et apres l'empilement/depilement)
%
%  Voir aussi SCID2D, ZCID2D
%