% SELECTION D'UNE PARTIE DE LA TEXTURE QR GENEREE PAR SCID2D OU ZCID2D
%
% Selection des points de la matrice Qr inclus dans une zone delimitee par
% une courbe spline fermee. On determine le plus petit rectangle dans lequel
% s'inscrit la zone et la matrice retournee a la taille de ce rectangle.
%
% Entrees :
%   -> points : points de controle de la spline
%   -> Qr     : matrice de texture
% Sorties :
%   -> Vr     : matrice de texture extraite :
%               Vr(i-rec_i,j-rec_j)=|Qr(i,j) si (i,j) est dans la zone selec-
%                                   |   tionnee ou (rec_i,rec_j) est le coin
%                                   |   superieur gauche du rectangle
%                                   |0 sinon
%
% Exemple :
%
%  % generation d'une texture
%  colormap(gray);
%  subplot(2,1,1);
%  x_max=10.23; y_max=7.67; r_min=0.02; dx=0.01;
%  choix_loi=2; params_loi=[2/3 0 .4]; motif=3; angle=1;
%  [Qr va zones]=scid2d(x_max,y_max,r_min,dx,choix_loi,params_loi,motif,angle);
%  NG=niveaux_gris(Qr);
%  imagesc(NG);
%
%  % forme de la zone a selectionner 
%  hold on
%  points=[388 146; 690 315; 400 559; 461 342; 388 146];
%  courbe=points_spline(points,50);
%  plot(courbe(:,2)',courbe(:,1)');
%  hold off
%  title('image de depart');
%
%  % selection de la partie de la texture
%  subplot(2,1,2);
%  Vr=selection(points,Qr);
%  NG=niveaux_gris(Vr);
%  imagesc(NG);
%  title('image selectionnee');
%
%  Voir aussi SCID2D, ZCID2D, SELECTION_INSERTION
%