% SYNTHESE DE CASCADES INFINIMENT DIVISIBLES EN 2D
% 
% Entrees :
%   -> x_max,y_max : taille de la matrice de texture a generer
%   -> r_min       : resolution de la cascade
%   -> dx          : pas d'espace
%   -> choix_loi   : 1, 2 ou 3 (voir ci-dessous)
%   -> params_loi  : parametres de la loi
%         -> choix_loi (params_loi)
%         -> 1 : F = Log-Normal distribution  (mu,sigma2,C)
%         -> 2 : F = exponential distribution (Temp,champ inutilise,C) 
%         -> 3 : F = Dirac (Wi=Cte=W0) (log-Poisson ou She-Leveque) 
%                    (W0<1,champ inutilise,C)
%   -> motif       : identifiant du motif
%         -> 1 : carre
%         -> 2 : cercle
%         -> 3 : utilisation d'une fonction lissante (cos^2(pi.r))
%         -> 4 : utilisation d'une fonction lissante (exp(-1/(1-(2r)^4))/exp(-1))
%         -> 5 : polygone
%         -> 6 : courbe spline fermee
%         -> 7 : ellipse
%         -> 8 : utilisation d'une fonction lissante quelconque
%   -> angle       : angle de rotation du motif (0:fixe,1:aleatoire)
%   -> parametres_motif : points du polygone, points de controle de la
%      spline (repeter le dernier point pour la spline) : [les x,les y] 
%      (x,y) est dans [-.5 .5]x[-.5 .5] ou [demi_grand_axe,foyer,angle]
%      d'une ellipse ou vecteur de la fonction lissante quelconque discretisee
%      sur [0;.5]
%      (parametre inutilise pour les autres motifs)
%                    
% Sorties :
%   -> Qr          : matrice de texture
%   -> va          : variables aleatoires utilisees pour la synthese
%   -> zones       : zones presentes dans la texture
%   -> x,y         : vecteurs position -> Qr(x,y) (facultatif)
%
% Exemple :
% 
%  x_max=10.23; y_max=7.67; r_min=0.02; dx=0.01;
%  choix_loi=2; params_loi=[2/3 0 1]; motif=5; angle=1;
%  polygone=.1*[-4 -3;-1 -1.5;1 -4;1 -1;4 0;1 1;1 4;-1 1.5;-4 3;-2 0];
%  [Qr va zones]=scid2d(x_max,y_max,r_min,dx,choix_loi,params_loi,motif,angle,polygone);
%  colormap(bone)
%  imagesc(Qr);
%
%  Voir aussi ZCID2D, NIVEAUX_GRIS
%