% CALCUL DES NIVEAUX DE GRIS DE L'IMAGE A AFFICHER
%
% Entrees :
%   -> Qr             : matrice de texture
%   -> ng_min, ng_max : extremum des niveaux de gris d'une image (action 2)
%   -> NG             : matrice des niveaux de gris (action 3)
%   -> cadre          : cadre du zoom (action 3)
%
% Sorties :
%   -> NG             : matrice des niveaux de gris (actions 1 et 2)
%   -> ng_min, ng_max : extremum des niveaux de gris d'une image (action 3)
%
% action 1 : repartion des niveaux de gris entre 0 et 255
% action 2 : repartion des niveaux de gris pour des extrema donnes
% action 3 : recherche des extrema des niveaux de gris d'une zone de l'image
%
% Exemple :
%
%  % generation d'une texture
%  colormap(gray);
%  subplot(2,1,1);
%  x_max=10.23; y_max=7.67; r_min=0.02; dx=0.01;
%  choix_loi=2; params_loi=[2/3 0 .4]; motif=3; angle=1;
%  [Qr,va,motifs]=scid2d(x_max,y_max,r_min,dx,choix_loi,params_loi,motif,angle);
%  NG=niveaux_gris(Qr);                                  % action 1
%  imagesc(NG);
%  title('image de depart');
%
%  % zoom dans la texture
%  subplot(2,1,2);
%  cadre=[0 0 x_max/2 y_max/2];
%  dx_prec=dx; dx=dx/2; r_min_prec=r_min; r_min=r_min/2;
%  [Qr,va,motifs]=zcid2d(cadre,dx,r_min,r_min_prec,va,motifs);
%  [ng_min,ng_max]=niveaux_gris(Qr,NG,cadre/dx_prec);    % action 3
%  NG=niveaux_gris(Qr,ng_min,ng_max);                    % action 2
%  imagesc(NG);
%  title('zoom sur le quart superieur gauche');
%  xlabel('en esperant que ce soit probant ...');
%
%  Voir aussi SCID2D, ZCID2D
%