close(texture)
clear all

% construction de scid2d

mex sources/scid2d.c sources/tirages.c sources/calculQr.c sources/spline.c sources/zones.c sources/global.c sources/rand_mersenne.c 


% construction de zcid2d

mex sources/zcid2d.c sources/tirages.c sources/calculQr.c sources/spline.c sources/zones.c sources/global.c sources/rand_mersenne.c 


% construction de pile

mex sources/pile.c


% construction de selection

mex sources/selection.c sources/spline.c


% construction de selection_insertion

mex sources/selection_insertion.c sources/zones.c sources/calculQr.c sources/spline.c


% construction de points_spline 

mex sources/points_spline.c sources/spline.c


% construction de niveaux_gris 

mex sources/niveaux_gris.c