
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     selectionner une partie d'une texture                   %                           
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function selection2(handles)

global Qr choix_mapping

   axes(handles.axes1);
   choix_mapping=0;
   afficher(Qr);
   courbe=[];
   bouton=0;
   i=1;
   hold on
   % tant que non clic droit on place les points
   while bouton~=3,
      [courbe(i,1),courbe(i,2),bouton]=ginput(1);
      plot(courbe(i,1),courbe(i,2),'go');
      drawnow;
      i=i+1;
   end
   % Construction de la courbe  et recuperation de la zone
   courbe(i,:)=courbe(1,:);
   pts=points_spline(courbe,200);
   plot(pts(:,1)',pts(:,2)','g');
   hold off;
   courbe=[courbe(:,2) courbe(:,1)];

   [Br]=selection(courbe,Qr);
 
   sauve=questdlg('Voulez vous gardez cette partie de texture pour pouvoir l integrer dans une autre texture?', ...
                  'Sauver la texture selectionee', ...
                  'Ok','Non','Ok');
   switch sauve,
      case 'Ok',
         [filename,pathname]=uiputfile({'*.mat', 'fichier MATLAB (*.mat)'; ...
                                       '*.*',   'tous les fichiers (*.*)'}, ...
                                       'Sauver les donnees de la texture selectionnee');
         if ~isequal(filename,0),
            chemin=sprintf('%s%s',pathname,filename);
            save(chemin,'Br');
         end  
   end
