% SELECTION D'UNE PARTIE DE LA TEXTURE GENEREE PAR SCID2D OU ZCID2D
% ET INSERTION DANS UNE AUTRE (EN UTILISANT LES VARIABLES ALEATOIRES)
%
% Entrees :
%   -> vaC      : variables aleatoires utilisees pour la synthese
%   -> zonesC   : zones presentes dans la texture courante
%   -> vaS      : variables aleatoires a inserer (insertion)
%   -> zonesS   : zones presentes dans la texture a inserer (insertion)
%   -> points   : points de controle de la spline (selection)
%
% Sorties :
%   -> va       : variables aleatoires utilisees pour la synthese
%   -> zones    : zones presentes dans la texture courante
%
% Exemple :
%
%  % generation d'une texture
%  colormap(bone);
%  subplot(2,1,1);
%  x_max=10.23; y_max=7.67; r_min=0.02; dx=0.01;
%  choix_loi=2; params_loi=[2/3 0 .4]; motif=4; angle=1;
%  [Qr,va,zones]=scid2d(x_max,y_max,r_min,dx,choix_loi,params_loi,motif,angle);
%  NG=niveaux_gris(Qr);
%  imagesc(NG);
%
%  % forme de la zone a selectionner
%  hold on
%  points=[388 146; 690 315; 400 559; 461 342; 388 146];
%  courbe=points_spline(points,50);
%  plot(courbe(:,1)',courbe(:,2)');
%  hold off
%  title('selection d une partie de la texture');
%
%  % selection de la partie de la texture
%  subplot(2,1,2);
%  [vaS,zonesS]=selection_insertion(va,zones,points*dx);
%
%  % generation d'une autre texture
%  choix_loi=1; params_loi=[-.75 .25 .2]; motif=5; angle=1;
%  polygone=.1*[-4 -3;-1 -1.5;1 -4;1 -1;4 0;1 1;1 4;-1 1.5;-4 3;-2 0];
%  [Qr,va,zones]=scid2d(x_max,y_max,r_min,dx,choix_loi,params_loi,motif,angle,polygone);
%
%  % insertion et recalcul de la texture
%  [va,zones]=selection_insertion(va,zones,vaS,zonesS);
%  cadre=[0,0,x_max,y_max];
%  [Qr va zones]=zcid2d(cadre,dx,r_min,r_min,va,zones);
%  NG=niveaux_gris(Qr);
%  imagesc(NG);
%  title('insertion dans une autre');
%
%  Voir aussi SCID2D, ZCID2D, SELECTION
%