% ZOOM DANS UNE CASCADE INFINIMENT DIVISIBLE EN 2D
% 
% Entrees :
%   -> cadre       : cadre de la zone de la texture a agrandir
%                    [coin superieur gauche (x et y);cote selon x;cote selon y]
%   -> dx          : pas d'espace
%   -> r_min       : resolution de la cascade
%   -> r_min_prec  : resolution precedente de la cascade
%   -> va          : variables aleatoires utilisees pour la synthese
%   -> zones       : zones presentes dans la texture
%
% Sorties :
%   -> Qr          : matrice de texture
%   -> va          : variables aleatoires utilisees pour la synthese
%   -> zones       : zones presentes dans la texture
%   -> x,y         : vecteurs position -> Qr(x,y) (facultatif)
%
% Exemple :
%
%  % generation d'une texture
%  colormap(gray);
%  subplot(2,1,1);
%  x_max=10.23; y_max=7.67; r_min=0.02; dx=0.01;
%  choix_loi=2; params_loi=[2/3 0 .4]; motif=3; angle=1;
%  [Qr,va,zones]=scid2d(x_max,y_max,r_min,dx,choix_loi,params_loi,motif,angle);
%  NG=niveaux_gris(Qr);
%  imagesc(NG);
%  title('image de depart');
%
%  % zoom dans la texture
%  subplot(2,1,2);
%  cadre=[0 0 x_max/2 y_max/2];
%  dx_prec=dx; dx=dx/2; r_min_prec=r_min; r_min=r_min/2;
%  [Qr va zones]=zcid2d(cadre,dx,r_min,r_min_prec,va,zones);
%  [ng_min,ng_max]=niveaux_gris(Qr,NG,cadre/dx_prec);
%  NG=niveaux_gris(Qr,ng_min,ng_max);
%  imagesc(NG);
%  title('zoom sur le quart superieur gauche');
%
%  Voir aussi SCID2D, PILE, NIVEAU_GRIS
%